import 'dart:convert';

extension JsonCodecExtensions on JsonCodec {
  /// 格式化Json
  ///
  /// ```dart
  /// json.format({"foo":"bar"});
  /// ```
  String format(Object obj) {
    return obj.toFormatJson;
  }
}

extension JsonExtensions on Object {
  /// 格式化Json
  ///
  /// ```dart
  /// {"foo":"bar"}.toFormatJson;
  /// ```
  String get toFormatJson {
    String deepSpace(int d) {
      String tab = ' ' * 2;
      return tab * d;
    }

    String returnSymbol = (StringBuffer()..writeln()).toString();

    String fmtJson(Object? obj,
        {int deep = 0, bool isObject = false, bool isReturn = false}) {
      StringBuffer buffer = StringBuffer();
      int nextDeep = deep + 1;
      if (obj is Map) {
        List list = obj.keys.toList();
        if (!isObject) buffer.write(deepSpace(deep));
        buffer.write("{");
        if (list.isEmpty) {
          buffer.write("}");
        } else {
          buffer
            ..writeln()
            ..writeAll(
                list.map((e) =>
                    "${deepSpace(nextDeep)}\"$e\": ${(fmtJson(obj[e], deep: nextDeep, isObject: true))}"),
                ",$returnSymbol")
            ..writeln()
            ..write(deepSpace(deep))
            ..write("}");
        }
      } else if (obj is List) {
        if (!isObject) buffer.write(deepSpace(deep));
        buffer.write("[");
        if (obj.isEmpty) {
          buffer.write("]");
        } else {
          buffer
            ..writeln()
            ..writeAll(
                obj.map((e) => fmtJson(e, deep: nextDeep, isReturn: true)),
                ",$returnSymbol")
            ..writeln()
            ..write(deepSpace(deep))
            ..write("]");
        }
      } else if (obj is String) {
        if (isReturn) buffer.write(deepSpace(deep));
        buffer.write(jsonEncode(obj));
      } else if (obj is num || obj is bool) {
        if (isReturn) buffer.write(deepSpace(deep));
        buffer.write(obj);
      } else {
        if (isReturn) buffer.write(deepSpace(deep));
        buffer.write("null");
      }
      return buffer.toString();
    }

    // return fmtJson(this);
    return fmtJson(jsonDecode(jsonEncode(this)));
  }
}
