extension StringExtensions on String {
  /// 首字母转换为大写
  ///
  /// ```dart
  /// // fooBar => FooBar
  /// ```
  String get toCapitalize {
    if (this.length < 2) return this.toUpperCase();
    return this[0].toUpperCase() + this.substring(1);
  }

  /// 转换为LowerCamel型式
  ///
  /// ```dart
  /// // FooBar => fooBar
  /// ```
  String get toLowerCamelCase {
    if (this.length < 2) return this.toLowerCase();
    return this[0].toLowerCase() + this.substring(1);
  }

  /// 转换为Kabab型式
  ///
  /// ```dart
  /// // fooBar => foo-bar
  /// ```
  String get toKababCase {
    return this.replaceAllMapped(RegExp('(.+?)([A-Z])'),
        (match) => '${match.group(1)}-${match.group(2)}'.toLowerCase());
  }

  /// 转换为Snake型式
  ///
  /// ```dart
  /// // fooBar => foo_bar
  /// ```
  String get toSnakeCase {
    return this
        .replaceAllMapped(RegExp('(.+?)([A-Z])'),
            (match) => '${match.group(1)}_${match.group(2)}')
        .toLowerCase();
  }

  /// 转换为大驼峰型式
  ///
  /// ```dart
  /// // foo_bar => FooBar
  /// ```
  String get toBigHump {
    return this.split(RegExp("[-_]")).map((e) => e.toCapitalize).join();
  }

  /// 转换为小驼峰型式
  ///
  /// ```dart
  /// // foo_bar => fooBar
  /// ```
  String get toLittleHump {
    return this
        .split(RegExp("[-_]"))
        .map((e) => e.toCapitalize)
        .join()
        .toLowerCamelCase;
  }
}
